@@ -32,6 +32,8 @@
 #import "OFHostAddressResolver.h"
 #import "OFNumber.h"
 #import "OFPair.h"
+#import "OFSocket.h"
+#import "OFSocket+Private.h"
 #import "OFString.h"
 #import "OFTCPSocket.h"
 #import "OFTimer.h"
@@ -183,7 +185,7 @@ parseResourceRecord(OFString *name, OFDNSClass DNSClass,
 
 		memset(&address, 0, sizeof(address));
 		address.family = OFSocketAddressFamilyIPv4;
-		address.length = sizeof(address.sockaddr.in);
+		address.length = (socklen_t)sizeof(address.sockaddr.in);
 
 		address.sockaddr.in.sin_family = AF_INET;
 		memcpy(&address.sockaddr.in.sin_addr.s_addr, buffer + i, 4);
@@ -370,7 +372,7 @@ parseResourceRecord(OFString *name, OFDNSClass DNSClass,
 
 		memset(&address, 0, sizeof(address));
 		address.family = OFSocketAddressFamilyIPv6;
-		address.length = sizeof(address.sockaddr.in6);
+		address.length = (socklen_t)sizeof(address.sockaddr.in6);
 
 #ifdef AF_INET6
 		address.sockaddr.in6.sin6_family = AF_INET6;
@@ -618,18 +620,6 @@ containsExpiredRecord(OFDNSResponseRecords responseRecords, uint32_t age)
 @end
 
 @implementation OFDNSResolver
-#ifdef OF_AMIGAOS
-+ (void)initialize
-{
-	if (self != [OFDNSResolver class])
-		return;
-
-	if (!_OFSocketInit())
-		@throw [OFInitializationFailedException
-		    exceptionWithClass: self];
-}
-#endif
-
 + (instancetype)resolver
 {
 	return [[[self alloc] init] autorelease];
@@ -640,6 +630,10 @@ containsExpiredRecord(OFDNSResponseRecords responseRecords, uint32_t age)
 	self = [super init];
 
 	@try {
+		if (!_OFSocketInit())
+			@throw [OFInitializationFailedException
+			    exceptionWithClass: self.class];
+
 		_settings = [[OFDNSResolverSettings alloc] init];
 		_queries = [[OFMutableDictionary alloc] init];
 		_TCPQueries = [[OFMutableDictionary alloc] init];
