@@ -40,7 +40,8 @@ OF_ASSUME_NONNULL_BEGIN
 #  define OF_FILE_MANAGER_SUPPORTS_SYMLINKS
 # endif
 # if defined(OF_LINUX) || defined(OF_MACOS) || defined(OF_FREEBSD) || \
-    defined(OF_NETBSD) || defined(OF_HAIKU) || defined(DOXYGEN)
+    defined(OF_NETBSD) || defined(OF_HAIKU) || defined(OF_SOLARIS) || \
+    defined(DOXYGEN)
 #  define OF_FILE_MANAGER_SUPPORTS_EXTENDED_ATTRIBUTES
 # endif
 #endif
@@ -263,7 +264,7 @@ extern const OFFileAttributeType OFFileTypeUnknown;
 #endif
 
 /**
- * @class OFFileManager OFFileManager.h ObjFW/OFFileManager.h
+ * @class OFFileManager OFFileManager.h ObjFW/ObjFW.h
  *
  * @brief A class which provides management for files, e.g. reading contents of
  *	  directories, deleting files, renaming files, etc.
