@@ -219,7 +219,7 @@
 				     length: length
 				       mode: runLoopMode
 # ifdef OF_HAVE_BLOCKS
-				      block: NULL
+				    handler: NULL
 # endif
 				   delegate: _delegate];
 }
@@ -243,7 +243,7 @@
 				exactLength: length
 				       mode: runLoopMode
 # ifdef OF_HAVE_BLOCKS
-				      block: NULL
+				    handler: NULL
 # endif
 				   delegate: _delegate];
 }
@@ -253,16 +253,47 @@
 		     length: (size_t)length
 		      block: (OFStreamAsyncReadBlock)block
 {
+	OFStreamReadHandler handler = ^ (OFStream *stream, void *buffer_,
+	    size_t length_, id exception) {
+		return block(length, exception);
+	};
+
 	[self asyncReadIntoBuffer: buffer
 			   length: length
 		      runLoopMode: OFDefaultRunLoopMode
-			    block: block];
+			  handler: handler];
+}
+
+- (void)asyncReadIntoBuffer: (void *)buffer
+		     length: (size_t)length
+		    handler: (OFStreamReadHandler)handler
+{
+	[self asyncReadIntoBuffer: buffer
+			   length: length
+		      runLoopMode: OFDefaultRunLoopMode
+			  handler: handler];
 }
 
 - (void)asyncReadIntoBuffer: (void *)buffer
 		     length: (size_t)length
 		runLoopMode: (OFRunLoopMode)runLoopMode
 		      block: (OFStreamAsyncReadBlock)block
+{
+	OFStreamReadHandler handler = ^ (OFStream *stream, void *buffer_,
+	    size_t length_, id exception) {
+		return block(length, exception);
+	};
+
+	[self asyncReadIntoBuffer: buffer
+			   length: length
+		      runLoopMode: runLoopMode
+			  handler: handler];
+}
+
+- (void)asyncReadIntoBuffer: (void *)buffer
+		     length: (size_t)length
+		runLoopMode: (OFRunLoopMode)runLoopMode
+		    handler: (OFStreamReadHandler)handler
 {
 	OFStream <OFReadyForReadingObserving> *stream =
 	    (OFStream <OFReadyForReadingObserving> *)self;
@@ -271,7 +302,7 @@
 				     buffer: buffer
 				     length: length
 				       mode: runLoopMode
-				      block: block
+				    handler: handler
 				   delegate: nil];
 }
 
@@ -279,16 +310,47 @@
 		exactLength: (size_t)length
 		      block: (OFStreamAsyncReadBlock)block
 {
+	OFStreamReadHandler handler = ^ (OFStream *stream, void *buffer_,
+	    size_t length_, id exception) {
+		return block(length, exception);
+	};
+
 	[self asyncReadIntoBuffer: buffer
 		      exactLength: length
 		      runLoopMode: OFDefaultRunLoopMode
-			    block: block];
+			  handler: handler];
+}
+
+- (void)asyncReadIntoBuffer: (void *)buffer
+		exactLength: (size_t)length
+		    handler: (OFStreamReadHandler)handler
+{
+	[self asyncReadIntoBuffer: buffer
+		      exactLength: length
+		      runLoopMode: OFDefaultRunLoopMode
+			  handler: handler];
 }
 
 - (void)asyncReadIntoBuffer: (void *)buffer
 		exactLength: (size_t)length
 		runLoopMode: (OFRunLoopMode)runLoopMode
 		      block: (OFStreamAsyncReadBlock)block
+{
+	OFStreamReadHandler handler = ^ (OFStream *stream, void *buffer_,
+	    size_t length_, id exception) {
+		return block(length, exception);
+	};
+
+	[self asyncReadIntoBuffer: buffer
+		      exactLength: length
+		      runLoopMode: runLoopMode
+			  handler: handler];
+}
+
+- (void)asyncReadIntoBuffer: (void *)buffer
+		exactLength: (size_t)length
+		runLoopMode: (OFRunLoopMode)runLoopMode
+		    handler: (OFStreamReadHandler)handler
 {
 	OFStream <OFReadyForReadingObserving> *stream =
 	    (OFStream <OFReadyForReadingObserving> *)self;
@@ -297,7 +359,7 @@
 				     buffer: buffer
 				exactLength: length
 				       mode: runLoopMode
-				      block: block
+				    handler: handler
 				   delegate: nil];
 }
 # endif
@@ -428,6 +490,22 @@
 	return data;
 }
 
+- (OFString *)readString
+{
+	return [self readStringWithEncoding: OFStringEncodingUTF8];
+}
+
+- (OFString *)readStringWithEncoding: (OFStringEncoding)encoding
+{
+	OFString *string = nil;
+
+	while ((string = [self tryReadStringWithEncoding: encoding]) == nil)
+		if (self.atEndOfStream)
+			return nil;
+
+	return string;
+}
+
 - (OFString *)readStringWithLength: (size_t)length
 {
 	return [self readStringWithLength: length
@@ -616,6 +694,33 @@
 }
 
 #ifdef OF_HAVE_SOCKETS
+- (void)asyncReadString
+{
+	[self asyncReadStringWithEncoding: OFStringEncodingUTF8
+			      runLoopMode: OFDefaultRunLoopMode];
+}
+
+- (void)asyncReadStringWithEncoding: (OFStringEncoding)encoding
+{
+	[self asyncReadStringWithEncoding: encoding
+			      runLoopMode: OFDefaultRunLoopMode];
+}
+
+- (void)asyncReadStringWithEncoding: (OFStringEncoding)encoding
+			runLoopMode: (OFRunLoopMode)runLoopMode
+{
+	OFStream <OFReadyForReadingObserving> *stream =
+	    (OFStream <OFReadyForReadingObserving> *)self;
+
+	[OFRunLoop of_addAsyncReadStringForStream: stream
+					 encoding: encoding
+					     mode: runLoopMode
+# ifdef OF_HAVE_BLOCKS
+					  handler: NULL
+# endif
+					 delegate: _delegate];
+}
+
 - (void)asyncReadLine
 {
 	[self asyncReadLineWithEncoding: OFStringEncodingUTF8
@@ -638,30 +743,98 @@
 				       encoding: encoding
 					   mode: runLoopMode
 # ifdef OF_HAVE_BLOCKS
-					  block: NULL
+					handler: NULL
 # endif
 				       delegate: _delegate];
 }
 
 # ifdef OF_HAVE_BLOCKS
+- (void)asyncReadStringWithHandler: (OFStreamStringReadHandler)handler
+{
+	[self asyncReadStringWithEncoding: OFStringEncodingUTF8
+			      runLoopMode: OFDefaultRunLoopMode
+				  handler: handler];
+}
+
+- (void)asyncReadStringWithEncoding: (OFStringEncoding)encoding
+			    handler: (OFStreamStringReadHandler)handler
+{
+	[self asyncReadStringWithEncoding: encoding
+			      runLoopMode: OFDefaultRunLoopMode
+				  handler: handler];
+}
+
+- (void)asyncReadStringWithEncoding: (OFStringEncoding)encoding
+			runLoopMode: (OFRunLoopMode)runLoopMode
+			    handler: (OFStreamStringReadHandler)handler
+{
+	OFStream <OFReadyForReadingObserving> *stream =
+	    (OFStream <OFReadyForReadingObserving> *)self;
+
+	[OFRunLoop of_addAsyncReadStringForStream: stream
+					 encoding: encoding
+					     mode: runLoopMode
+					  handler: handler
+					 delegate: nil];
+}
+
 - (void)asyncReadLineWithBlock: (OFStreamAsyncReadLineBlock)block
+{
+	OFStreamStringReadHandler handler = ^ (OFStream *stream,
+	    OFString *string, id exception) {
+		return block(string, exception);
+	};
+
+	[self asyncReadLineWithEncoding: OFStringEncodingUTF8
+			    runLoopMode: OFDefaultRunLoopMode
+				handler: handler];
+}
+
+- (void)asyncReadLineWithHandler: (OFStreamStringReadHandler)handler
 {
 	[self asyncReadLineWithEncoding: OFStringEncodingUTF8
 			    runLoopMode: OFDefaultRunLoopMode
-				  block: block];
+				handler: handler];
 }
 
 - (void)asyncReadLineWithEncoding: (OFStringEncoding)encoding
 			    block: (OFStreamAsyncReadLineBlock)block
 {
+	OFStreamStringReadHandler handler = ^ (OFStream *stream,
+	    OFString *string, id exception) {
+		return block(string, exception);
+	};
+
 	[self asyncReadLineWithEncoding: encoding
 			    runLoopMode: OFDefaultRunLoopMode
-				  block: block];
+				handler: handler];
+}
+
+- (void)asyncReadLineWithEncoding: (OFStringEncoding)encoding
+			  handler: (OFStreamStringReadHandler)handler
+{
+	[self asyncReadLineWithEncoding: encoding
+			    runLoopMode: OFDefaultRunLoopMode
+				handler: handler];
 }
 
 - (void)asyncReadLineWithEncoding: (OFStringEncoding)encoding
 		      runLoopMode: (OFRunLoopMode)runLoopMode
 			    block: (OFStreamAsyncReadLineBlock)block
+{
+	OFStreamStringReadHandler handler = ^ (OFStream *stream,
+	    OFString *string, id exception) {
+		return block(string, exception);
+	};
+
+	[self asyncReadLineWithEncoding: encoding
+			    runLoopMode: runLoopMode
+				handler: handler];
+}
+
+- (void)asyncReadLineWithEncoding: (OFStringEncoding)encoding
+		      runLoopMode: (OFRunLoopMode)runLoopMode
+			  handler: (OFStreamStringReadHandler)handler
 {
 	OFStream <OFReadyForReadingObserving> *stream =
 	    (OFStream <OFReadyForReadingObserving> *)self;
@@ -669,12 +842,150 @@
 	[OFRunLoop of_addAsyncReadLineForStream: stream
 				       encoding: encoding
 					   mode: runLoopMode
-					  block: block
+					handler: handler
 				       delegate: nil];
 }
 # endif
 #endif
 
+- (OFString *)tryReadString
+{
+	return [self tryReadStringWithEncoding: OFStringEncodingUTF8];
+}
+
+- (OFString *)tryReadStringWithEncoding: (OFStringEncoding)encoding
+{
+	size_t pageSize, bufferLength;
+	char *buffer, *readBuffer;
+	OFString *ret;
+
+	/* Look if there's something in our buffer */
+	if (!_waitingForDelimiter && _readBuffer != NULL) {
+		for (size_t i = 0; i < _readBufferLength; i++) {
+			if (_readBuffer[i] == '\0') {
+				ret = [OFString
+				    stringWithCString: _readBuffer
+					     encoding: encoding
+					       length: i];
+
+				_readBuffer += i + 1;
+				_readBufferLength -= i + 1;
+
+				_waitingForDelimiter = false;
+				return ret;
+			}
+		}
+	}
+
+	/* Read and see if we got a \0 */
+	pageSize = [OFSystemInfo pageSize];
+	buffer = OFAllocMemory(1, pageSize);
+
+	@try {
+		if ([self lowlevelIsAtEndOfStream]) {
+			if (_readBuffer == NULL) {
+				_waitingForDelimiter = false;
+				return nil;
+			}
+
+			ret = [OFString stringWithCString: _readBuffer
+						 encoding: encoding
+						   length: _readBufferLength];
+
+			OFFreeMemory(_readBufferMemory);
+			_readBuffer = _readBufferMemory = NULL;
+			_readBufferLength = 0;
+
+			_waitingForDelimiter = false;
+			return ret;
+		}
+
+		bufferLength = [self lowlevelReadIntoBuffer: buffer
+						     length: pageSize];
+
+		/* Look if there's a \0 */
+		for (size_t i = 0; i < bufferLength; i++) {
+			if (buffer[i] == '\0') {
+				size_t retLength;
+				char *retCString;
+
+				retLength = _readBufferLength + i;
+				retCString = OFAllocMemory(retLength, 1);
+
+				memcpy(retCString, _readBuffer,
+				    _readBufferLength);
+				memcpy(retCString + _readBufferLength,
+				    buffer, i);
+
+				@try {
+					ret = [OFString
+					    stringWithCString: retCString
+						     encoding: encoding
+						       length: retLength];
+				} @catch (id e) {
+					if (bufferLength > 0) {
+						/*
+						 * Append data to _readBuffer
+						 * to prevent loss of data.
+						 */
+						readBuffer = OFAllocMemory(
+						    _readBufferLength +
+						    bufferLength, 1);
+
+						memcpy(readBuffer, _readBuffer,
+						    _readBufferLength);
+						memcpy(readBuffer +
+						    _readBufferLength,
+						    buffer, bufferLength);
+
+						OFFreeMemory(_readBufferMemory);
+						_readBuffer = readBuffer;
+						_readBufferMemory = readBuffer;
+						_readBufferLength +=
+						    bufferLength;
+					}
+
+					@throw e;
+				} @finally {
+					OFFreeMemory(retCString);
+				}
+
+				readBuffer = OFAllocMemory(bufferLength - i - 1,
+				    1);
+				if (readBuffer != NULL)
+					memcpy(readBuffer, buffer + i + 1,
+					    bufferLength - i - 1);
+
+				OFFreeMemory(_readBufferMemory);
+				_readBuffer = _readBufferMemory = readBuffer;
+				_readBufferLength = bufferLength - i - 1;
+
+				_waitingForDelimiter = false;
+				return ret;
+			}
+		}
+
+		/* No \0 was found */
+		if (bufferLength > 0) {
+			readBuffer = OFAllocMemory(
+			    _readBufferLength + bufferLength, 1);
+
+			memcpy(readBuffer, _readBuffer, _readBufferLength);
+			memcpy(readBuffer + _readBufferLength,
+			    buffer, bufferLength);
+
+			OFFreeMemory(_readBufferMemory);
+			_readBuffer = _readBufferMemory = readBuffer;
+			_readBufferLength += bufferLength;
+		}
+	} @finally {
+		OFFreeMemory(buffer);
+	}
+
+	_waitingForDelimiter = true;
+	return nil;
+}
+
 - (OFString *)tryReadLine
 {
 	return [self tryReadLineWithEncoding: OFStringEncodingUTF8];
@@ -841,7 +1152,6 @@
 	return nil;
 }
 
-
 - (OFString *)readUntilDelimiter: (OFString *)delimiter
 {
 	return [self readUntilDelimiter: delimiter
@@ -941,7 +1251,7 @@
 					data: data
 					mode: runLoopMode
 # ifdef OF_HAVE_BLOCKS
-				       block: NULL
+				     handler: NULL
 # endif
 				    delegate: _delegate];
 }
@@ -973,7 +1283,7 @@
 				    encoding: encoding
 					mode: runLoopMode
 # ifdef OF_HAVE_BLOCKS
-				       block: NULL
+				     handler: NULL
 # endif
 				    delegate: _delegate];
 }
@@ -981,14 +1291,41 @@
 # ifdef OF_HAVE_BLOCKS
 - (void)asyncWriteData: (OFData *)data block: (OFStreamAsyncWriteDataBlock)block
 {
+	OFStreamDataWrittenHandler handler = ^ (OFStream *stream, OFData *data_,
+	    size_t bytesWritten, id exception) {
+		return block(bytesWritten, exception);
+	};
+
 	[self asyncWriteData: data
 		 runLoopMode: OFDefaultRunLoopMode
-		       block: block];
+		     handler: handler];
+}
+
+- (void)asyncWriteData: (OFData *)data
+	       handler: (OFStreamDataWrittenHandler)handler
+{
+	[self asyncWriteData: data
+		 runLoopMode: OFDefaultRunLoopMode
+		     handler: handler];
 }
 
 - (void)asyncWriteData: (OFData *)data
 	   runLoopMode: (OFRunLoopMode)runLoopMode
 		 block: (OFStreamAsyncWriteDataBlock)block
+{
+	OFStreamDataWrittenHandler handler = ^ (OFStream *stream, OFData *data_,
+	    size_t bytesWritten, id exception) {
+		return block(bytesWritten, exception);
+	};
+
+	[self asyncWriteData: data
+		 runLoopMode: runLoopMode
+		     handler: handler];
+}
+
+- (void)asyncWriteData: (OFData *)data
+	   runLoopMode: (OFRunLoopMode)runLoopMode
+	       handler: (OFStreamDataWrittenHandler)handler
 {
 	OFStream <OFReadyForWritingObserving> *stream =
 	    (OFStream <OFReadyForWritingObserving> *)self;
@@ -996,33 +1333,81 @@
 	[OFRunLoop of_addAsyncWriteForStream: stream
 					data: data
 					mode: runLoopMode
-				       block: block
+				     handler: handler
 				    delegate: nil];
 }
 
 - (void)asyncWriteString: (OFString *)string
 		   block: (OFStreamAsyncWriteStringBlock)block
 {
+	OFStreamStringWrittenHandler handler = ^ (OFStream *stream,
+	    OFString *string_, OFStringEncoding encoding_, size_t bytesWritten,
+	    id exception) {
+		return block(bytesWritten, exception);
+	};
+
 	[self asyncWriteString: string
 		      encoding: OFStringEncodingUTF8
 		   runLoopMode: OFDefaultRunLoopMode
-			 block: block];
+		       handler: handler];
+}
+
+- (void)asyncWriteString: (OFString *)string
+		 handler: (OFStreamStringWrittenHandler)handler
+{
+	[self asyncWriteString: string
+		      encoding: OFStringEncodingUTF8
+		   runLoopMode: OFDefaultRunLoopMode
+		       handler: handler];
 }
 
 - (void)asyncWriteString: (OFString *)string
 		encoding: (OFStringEncoding)encoding
 		   block: (OFStreamAsyncWriteStringBlock)block
 {
+	OFStreamStringWrittenHandler handler = ^ (OFStream *stream,
+	    OFString *string_, OFStringEncoding encoding_, size_t bytesWritten,
+	    id exception) {
+		return block(bytesWritten, exception);
+	};
+
 	[self asyncWriteString: string
 		      encoding: encoding
 		   runLoopMode: OFDefaultRunLoopMode
-			 block: block];
+		       handler: handler];
+}
+
+- (void)asyncWriteString: (OFString *)string
+		encoding: (OFStringEncoding)encoding
+		 handler: (OFStreamStringWrittenHandler)handler
+{
+	[self asyncWriteString: string
+		      encoding: encoding
+		   runLoopMode: OFDefaultRunLoopMode
+		       handler: handler];
 }
 
 - (void)asyncWriteString: (OFString *)string
 		encoding: (OFStringEncoding)encoding
 	     runLoopMode: (OFRunLoopMode)runLoopMode
 		   block: (OFStreamAsyncWriteStringBlock)block
+{
+	OFStreamStringWrittenHandler handler = ^ (OFStream *stream,
+	    OFString *string_, OFStringEncoding encoding_, size_t bytesWritten,
+	    id exception) {
+		return block(bytesWritten, exception);
+	};
+
+	[self asyncWriteString: string
+		      encoding: encoding
+		   runLoopMode: runLoopMode
+		       handler: handler];
+}
+
+- (void)asyncWriteString: (OFString *)string
+		encoding: (OFStringEncoding)encoding
+	     runLoopMode: (OFRunLoopMode)runLoopMode
+		 handler: (OFStreamStringWrittenHandler)handler
 {
 	OFStream <OFReadyForWritingObserving> *stream =
 	    (OFStream <OFReadyForWritingObserving> *)self;
@@ -1031,7 +1416,7 @@
 				      string: string
 				    encoding: encoding
 					mode: runLoopMode
-				       block: block
+				     handler: handler
 				    delegate: nil];
 }
 # endif
