@@ -34,6 +34,7 @@
 #endif
 
 #import "OFString.h"
+#import "OFString+Private.h"
 #import "OFASPrintF.h"
 #import "OFArray.h"
 #import "OFCharacterSet.h"
@@ -48,6 +49,7 @@
 #import "OFLocale.h"
 #import "OFStream.h"
 #import "OFSystemInfo.h"
+#import "OFTaggedPointerString.h"
 #import "OFUTF8String.h"
 #import "OFUTF8String+Private.h"
 
@@ -89,13 +91,23 @@ static struct {
 static locale_t cLocale;
 #endif
 
+#ifdef OF_OBJFW_RUNTIME
+# if UINTPTR_MAX == UINT64_MAX
+#  define MAX_TAGGED_POINTER_LENGTH 8
+# else
+#  define MAX_TAGGED_POINTER_LENGTH 4
+# endif
+#endif
+
 @interface OFString ()
 - (size_t)of_getCString: (char *)cString
 	      maxLength: (size_t)maxLength
 	       encoding: (OFStringEncoding)encoding
-		  lossy: (bool)lossy OF_DIRECT;
+		  lossy: (bool)lossy
+	       insecure: (bool)insecure OF_DIRECT;
 - (const char *)of_cStringWithEncoding: (OFStringEncoding)encoding
-				 lossy: (bool)lossy OF_DIRECT;
+				 lossy: (bool)lossy
+			      insecure: (bool)insecure OF_DIRECT;
 - (OFString *)
     of_JSONRepresentationWithOptions: (OFJSONRepresentationOptions)options
 			       depth: (size_t)depth;
@@ -105,27 +117,31 @@ static locale_t cLocale;
 @end
 
 extern bool _OFUnicodeToISO8859_2(const OFUnichar *, unsigned char *,
-    size_t, bool);
+    size_t, bool, bool);
 extern bool _OFUnicodeToISO8859_3(const OFUnichar *, unsigned char *,
-    size_t, bool);
+    size_t, bool, bool);
 extern bool _OFUnicodeToISO8859_15(const OFUnichar *, unsigned char *,
-    size_t, bool);
+    size_t, bool, bool);
+extern bool _OFUnicodeToWindows1250(const OFUnichar *, unsigned char *,
+    size_t, bool, bool);
 extern bool _OFUnicodeToWindows1251(const OFUnichar *, unsigned char *,
-    size_t, bool);
+    size_t, bool, bool);
 extern bool _OFUnicodeToWindows1252(const OFUnichar *, unsigned char *,
-    size_t, bool);
+    size_t, bool, bool);
 extern bool _OFUnicodeToCodepage437(const OFUnichar *, unsigned char *,
-    size_t, bool);
+    size_t, bool, bool);
 extern bool _OFUnicodeToCodepage850(const OFUnichar *, unsigned char *,
-    size_t, bool);
+    size_t, bool, bool);
+extern bool _OFUnicodeToCodepage852(const OFUnichar *, unsigned char *,
+    size_t, bool, bool);
 extern bool _OFUnicodeToCodepage858(const OFUnichar *, unsigned char *,
-    size_t, bool);
+    size_t, bool, bool);
 extern bool _OFUnicodeToMacRoman(const OFUnichar *, unsigned char *,
-    size_t, bool);
+    size_t, bool, bool);
 extern bool _OFUnicodeToKOI8R(const OFUnichar *, unsigned char *,
-    size_t, bool);
+    size_t, bool, bool);
 extern bool _OFUnicodeToKOI8U(const OFUnichar *, unsigned char *,
-    size_t, bool);
+    size_t, bool, bool);
 
 /* References for static linking */
 void OF_VISIBILITY_HIDDEN
@@ -172,6 +188,10 @@ OFStringEncodingParseName(OFString *string)
 	else if ([string isEqual: @"iso-8859-15"] ||
 	    [string isEqual: @"iso_8859-15"])
 		encoding = OFStringEncodingISO8859_15;
+	else if ([string isEqual: @"windows-1250"] ||
+	    [string isEqual: @"cp1250"] || [string isEqual: @"cp-1250"] ||
+	    [string isEqual: @"1250"])
+		encoding = OFStringEncodingWindows1250;
 	else if ([string isEqual: @"windows-1251"] ||
 	    [string isEqual: @"cp1251"] || [string isEqual: @"cp-1251"] ||
 	    [string isEqual: @"1251"])
@@ -186,6 +206,9 @@ OFStringEncodingParseName(OFString *string)
 	else if ([string isEqual: @"cp850"] || [string isEqual: @"cp-850"] ||
 	    [string isEqual: @"ibm850"] || [string isEqual: @"850"])
 		encoding = OFStringEncodingCodepage850;
+	else if ([string isEqual: @"cp852"] || [string isEqual: @"cp-852"] ||
+	    [string isEqual: @"ibm852"] || [string isEqual: @"852"])
+		encoding = OFStringEncodingCodepage852;
 	else if ([string isEqual: @"cp858"] || [string isEqual: @"cp-858"] ||
 	    [string isEqual: @"ibm858"] || [string isEqual: @"858"])
 		encoding = OFStringEncodingCodepage858;
@@ -219,6 +242,8 @@ OFStringEncodingName(OFStringEncoding encoding)
 		return @"ISO 8859-3";
 	case OFStringEncodingISO8859_15:
 		return @"ISO 8859-15";
+	case OFStringEncodingWindows1250:
+		return @"Windows-1250";
 	case OFStringEncodingWindows1251:
 		return @"Windows-1251";
 	case OFStringEncodingWindows1252:
@@ -227,6 +252,8 @@ OFStringEncodingName(OFStringEncoding encoding)
 		return @"Codepage 437";
 	case OFStringEncodingCodepage850:
 		return @"Codepage 850";
+	case OFStringEncodingCodepage852:
+		return @"Codepage 852";
 	case OFStringEncodingCodepage858:
 		return @"Codepage 858";
 	case OFStringEncodingMacRoman:
@@ -350,6 +377,24 @@ _OFStrDup(const char *string)
 	return copy;
 }
 
+#ifdef OF_OBJFW_RUNTIME
+static bool
+isASCIIWithoutNull(const char *string, size_t length)
+{
+	uint8_t combined = 0;
+	bool containsNull = false;
+
+	for (size_t i = 0; i < length; i++) {
+		combined |= string[i];
+
+		if (string[i] == '\0')
+			containsNull = true;
+	}
+
+	return !(combined & ~0x7F) && !containsNull;
+}
+#endif
+
 @implementation OFPlaceholderString
 #ifdef __clang__
 /* We intentionally don't call into super, so silence the warning. */
@@ -359,16 +404,30 @@ _OFStrDup(const char *string)
 #endif
 - (instancetype)init
 {
-	return (id)[[OFUTF8String alloc] init];
+	return (id)@"";
 }
 
 - (instancetype)initWithUTF8String: (const char *)UTF8String
 {
+	size_t length = strlen(UTF8String);
 	OFUTF8String *string;
-	size_t length;
 	void *storage;
 
-	length = strlen(UTF8String);
+	if (length == 0)
+		return (id)@"";
+
+#ifdef OF_OBJFW_RUNTIME
+	if (length <= MAX_TAGGED_POINTER_LENGTH &&
+	    isASCIIWithoutNull(UTF8String, length)) {
+		id ret = [OFTaggedPointerString
+		    stringWithASCIIString: UTF8String
+				   length: length];
+
+		if (ret != nil)
+			return ret;
+	}
+#endif
+
 	string = OFAllocObject([OFUTF8String class], length + 1, 1, &storage);
 
 	return (id)[string of_initWithUTF8String: UTF8String
@@ -382,6 +441,21 @@ _OFStrDup(const char *string)
 	OFUTF8String *string;
 	void *storage;
 
+	if (UTF8StringLength == 0)
+		return (id)@"";
+
+#ifdef OF_OBJFW_RUNTIME
+	if (UTF8StringLength <= MAX_TAGGED_POINTER_LENGTH &&
+	    isASCIIWithoutNull(UTF8String, UTF8StringLength)) {
+		id ret = [OFTaggedPointerString
+		    stringWithASCIIString: UTF8String
+				   length: UTF8StringLength];
+
+		if (ret != nil)
+			return ret;
+	}
+#endif
+
 	string = OFAllocObject([OFUTF8String class], UTF8StringLength + 1, 1,
 	    &storage);
 
@@ -411,12 +485,27 @@ _OFStrDup(const char *string)
 - (instancetype)initWithCString: (const char *)cString
 		       encoding: (OFStringEncoding)encoding
 {
-	if (encoding == OFStringEncodingUTF8) {
+	if (encoding == OFStringEncodingUTF8 ||
+	    encoding == OFStringEncodingASCII) {
+		size_t length = strlen(cString);
 		OFUTF8String *string;
-		size_t length;
 		void *storage;
 
-		length = strlen(cString);
+		if (length == 0)
+			return (id)@"";
+
+#ifdef OF_OBJFW_RUNTIME
+		if (length <= MAX_TAGGED_POINTER_LENGTH &&
+		    isASCIIWithoutNull(cString, length)) {
+			id ret = [OFTaggedPointerString
+			    stringWithASCIIString: cString
+					   length: length];
+
+			if (ret != nil)
+				return ret;
+		}
+#endif
+
 		string = OFAllocObject([OFUTF8String class], length + 1, 1,
 		    &storage);
 
@@ -433,10 +522,26 @@ _OFStrDup(const char *string)
 		       encoding: (OFStringEncoding)encoding
 			 length: (size_t)cStringLength
 {
-	if (encoding == OFStringEncodingUTF8) {
+	if (encoding == OFStringEncodingUTF8 ||
+	    encoding == OFStringEncodingASCII) {
 		OFUTF8String *string;
 		void *storage;
 
+		if (cStringLength == 0)
+			return (id)@"";
+
+#ifdef OF_OBJFW_RUNTIME
+		if (cStringLength <= MAX_TAGGED_POINTER_LENGTH &&
+		    isASCIIWithoutNull(cString, cStringLength)) {
+			id ret = [OFTaggedPointerString
+			    stringWithASCIIString: cString
+					   length: cStringLength];
+
+			if (ret != nil)
+				return ret;
+		}
+#endif
+
 		string = OFAllocObject([OFUTF8String class], cStringLength + 1,
 		    1, &storage);
 
@@ -465,6 +570,34 @@ _OFStrDup(const char *string)
 - (instancetype)initWithCharacters: (const OFUnichar *)string
 			    length: (size_t)length
 {
+	if (length == 0)
+		return (id)@"";
+
+#ifdef OF_OBJFW_RUNTIME
+	if (length <= MAX_TAGGED_POINTER_LENGTH) {
+		char buffer[MAX_TAGGED_POINTER_LENGTH];
+		bool useTaggedPointer = true;
+
+		for (size_t i = 0; i < length; i++) {
+			if (string[i] >= 0x80 || string[i] == 0) {
+				useTaggedPointer = false;
+				break;
+			}
+
+			buffer[i] = (char)string[i];
+		}
+
+		if (useTaggedPointer) {
+			id ret = [OFTaggedPointerString
+			    stringWithASCIIString: buffer
+					   length: length];
+
+			if (ret != nil)
+				return ret;
+		}
+	}
+#endif
+
 	return (id)[[OFUTF8String alloc] initWithCharacters: string
 						     length: length];
 }
@@ -1052,6 +1185,7 @@ OF_SINGLETON_METHODS
 	      maxLength: (size_t)maxLength
 	       encoding: (OFStringEncoding)encoding
 		  lossy: (bool)lossy
+	       insecure: (bool)insecure
 {
 	const OFUnichar *characters = self.characters;
 	size_t i, length = self.length;
@@ -1062,7 +1196,12 @@ OF_SINGLETON_METHODS
 
 		for (i = 0; i < length; i++) {
 			char buffer[4];
-			size_t len = _OFUTF8StringEncode(characters[i], buffer);
+			size_t len;
+
+			if OF_UNLIKELY (!insecure && characters[i] == 0)
+				@throw [OFInvalidEncodingException exception];
+
+			len = _OFUTF8StringEncode(characters[i], buffer);
 
 			/*
 			 * Check for one more than the current index, as we
@@ -1098,6 +1237,9 @@ OF_SINGLETON_METHODS
 			@throw [OFOutOfRangeException exception];
 
 		for (i = 0; i < length; i++) {
+			if OF_UNLIKELY (!insecure && characters[i] == 0)
+				@throw [OFInvalidEncodingException exception];
+
 			if OF_UNLIKELY (characters[i] > 0x80) {
 				if (lossy)
 					cString[i] = '?';
@@ -1116,6 +1258,9 @@ OF_SINGLETON_METHODS
 			@throw [OFOutOfRangeException exception];
 
 		for (i = 0; i < length; i++) {
+			if OF_UNLIKELY (!insecure && characters[i] == 0)
+				@throw [OFInvalidEncodingException exception];
+
 			if OF_UNLIKELY (characters[i] > 0xFF) {
 				if (lossy)
 					cString[i] = '?';
@@ -1135,7 +1280,7 @@ OF_SINGLETON_METHODS
 			@throw [OFOutOfRangeException exception];
 
 		if (!_OFUnicodeToISO8859_2(characters, (unsigned char *)cString,
-		    length, lossy))
+		    length, lossy, insecure))
 			@throw [OFInvalidEncodingException exception];
 
 		cString[length] = '\0';
@@ -1148,7 +1293,7 @@ OF_SINGLETON_METHODS
 			@throw [OFOutOfRangeException exception];
 
 		if (!_OFUnicodeToISO8859_3(characters, (unsigned char *)cString,
-		    length, lossy))
+		    length, lossy, insecure))
 			@throw [OFInvalidEncodingException exception];
 
 		cString[length] = '\0';
@@ -1161,7 +1306,20 @@ OF_SINGLETON_METHODS
 			@throw [OFOutOfRangeException exception];
 
 		if (!_OFUnicodeToISO8859_15(characters,
-		    (unsigned char *)cString, length, lossy))
+		    (unsigned char *)cString, length, lossy, insecure))
+			@throw [OFInvalidEncodingException exception];
+
+		cString[length] = '\0';
+
+		return length;
+#endif
+#ifdef HAVE_WINDOWS_1250
+	case OFStringEncodingWindows1250:
+		if (length + 1 > maxLength)
+			@throw [OFOutOfRangeException exception];
+
+		if (!_OFUnicodeToWindows1250(characters,
+		    (unsigned char *)cString, length, lossy, insecure))
 			@throw [OFInvalidEncodingException exception];
 
 		cString[length] = '\0';
@@ -1174,7 +1332,7 @@ OF_SINGLETON_METHODS
 			@throw [OFOutOfRangeException exception];
 
 		if (!_OFUnicodeToWindows1251(characters,
-		    (unsigned char *)cString, length, lossy))
+		    (unsigned char *)cString, length, lossy, insecure))
 			@throw [OFInvalidEncodingException exception];
 
 		cString[length] = '\0';
@@ -1187,7 +1345,7 @@ OF_SINGLETON_METHODS
 			@throw [OFOutOfRangeException exception];
 
 		if (!_OFUnicodeToWindows1252(characters,
-		    (unsigned char *)cString, length, lossy))
+		    (unsigned char *)cString, length, lossy, insecure))
 			@throw [OFInvalidEncodingException exception];
 
 		cString[length] = '\0';
@@ -1200,7 +1358,7 @@ OF_SINGLETON_METHODS
 			@throw [OFOutOfRangeException exception];
 
 		if (!_OFUnicodeToCodepage437(characters,
-		    (unsigned char *)cString, length, lossy))
+		    (unsigned char *)cString, length, lossy, insecure))
 			@throw [OFInvalidEncodingException exception];
 
 		cString[length] = '\0';
@@ -1213,7 +1371,20 @@ OF_SINGLETON_METHODS
 			@throw [OFOutOfRangeException exception];
 
 		if (!_OFUnicodeToCodepage850(characters,
-		    (unsigned char *)cString, length, lossy))
+		    (unsigned char *)cString, length, lossy, insecure))
+			@throw [OFInvalidEncodingException exception];
+
+		cString[length] = '\0';
+
+		return length;
+#endif
+#ifdef HAVE_CODEPAGE_852
+	case OFStringEncodingCodepage852:
+		if (length + 1 > maxLength)
+			@throw [OFOutOfRangeException exception];
+
+		if (!_OFUnicodeToCodepage852(characters,
+		    (unsigned char *)cString, length, lossy, insecure))
 			@throw [OFInvalidEncodingException exception];
 
 		cString[length] = '\0';
@@ -1226,7 +1397,7 @@ OF_SINGLETON_METHODS
 			@throw [OFOutOfRangeException exception];
 
 		if (!_OFUnicodeToCodepage858(characters,
-		    (unsigned char *)cString, length, lossy))
+		    (unsigned char *)cString, length, lossy, insecure))
 			@throw [OFInvalidEncodingException exception];
 
 		cString[length] = '\0';
@@ -1239,7 +1410,7 @@ OF_SINGLETON_METHODS
 			@throw [OFOutOfRangeException exception];
 
 		if (!_OFUnicodeToMacRoman(characters, (unsigned char *)cString,
-		    length, lossy))
+		    length, lossy, insecure))
 			@throw [OFInvalidEncodingException exception];
 
 		cString[length] = '\0';
@@ -1252,7 +1423,7 @@ OF_SINGLETON_METHODS
 			@throw [OFOutOfRangeException exception];
 
 		if (!_OFUnicodeToKOI8R(characters, (unsigned char *)cString,
-		    length, lossy))
+		    length, lossy, insecure))
 			@throw [OFInvalidEncodingException exception];
 
 		cString[length] = '\0';
@@ -1265,7 +1436,7 @@ OF_SINGLETON_METHODS
 			@throw [OFOutOfRangeException exception];
 
 		if (!_OFUnicodeToKOI8U(characters, (unsigned char *)cString,
-		    length, lossy))
+		    length, lossy, insecure))
 			@throw [OFInvalidEncodingException exception];
 
 		cString[length] = '\0';
@@ -1284,7 +1455,8 @@ OF_SINGLETON_METHODS
 	return [self of_getCString: cString
 			 maxLength: maxLength
 			  encoding: encoding
-			     lossy: false];
+			     lossy: false
+			  insecure: false];
 }
 
 - (size_t)getLossyCString: (char *)cString
@@ -1294,11 +1466,13 @@ OF_SINGLETON_METHODS
 	return [self of_getCString: cString
 			 maxLength: maxLength
 			  encoding: encoding
-			     lossy: true];
+			     lossy: true
+			  insecure: false];
 }
 
 - (const char *)of_cStringWithEncoding: (OFStringEncoding)encoding
 				 lossy: (bool)lossy
+			      insecure: (bool)insecure
 {
 	size_t length = self.length;
 	char *cString;
@@ -1314,7 +1488,8 @@ OF_SINGLETON_METHODS
 			    of_getCString: cString
 				maxLength: (length * 4) + 1
 				 encoding: OFStringEncodingUTF8
-				    lossy: lossy];
+				    lossy: lossy
+				 insecure: insecure];
 		} @catch (id e) {
 			OFFreeMemory(cString);
 			@throw e;
@@ -1332,10 +1507,12 @@ OF_SINGLETON_METHODS
 	case OFStringEncodingISO8859_2:
 	case OFStringEncodingISO8859_3:
 	case OFStringEncodingISO8859_15:
+	case OFStringEncodingWindows1250:
 	case OFStringEncodingWindows1251:
 	case OFStringEncodingWindows1252:
 	case OFStringEncodingCodepage437:
 	case OFStringEncodingCodepage850:
+	case OFStringEncodingCodepage852:
 	case OFStringEncodingCodepage858:
 	case OFStringEncodingMacRoman:
 	case OFStringEncodingKOI8R:
@@ -1346,7 +1523,8 @@ OF_SINGLETON_METHODS
 			cStringLength = [self of_getCString: cString
 						  maxLength: length + 1
 						   encoding: encoding
-						      lossy: lossy];
+						      lossy: lossy
+						   insecure: insecure];
 		} @catch (id e) {
 			OFFreeMemory(cString);
 			@throw e;
@@ -1371,17 +1549,30 @@ OF_SINGLETON_METHODS
 
 - (const char *)cStringWithEncoding: (OFStringEncoding)encoding
 {
-	return [self of_cStringWithEncoding: encoding lossy: false];
+	return [self of_cStringWithEncoding: encoding
+				      lossy: false
+				   insecure: false];
 }
 
 - (const char *)lossyCStringWithEncoding: (OFStringEncoding)encoding
 {
-	return [self of_cStringWithEncoding: encoding lossy: true];
+	return [self of_cStringWithEncoding: encoding
+				      lossy: true
+				   insecure: false];
+}
+
+- (const char *)insecureCStringWithEncoding: (OFStringEncoding)encoding
+{
+	return [self of_cStringWithEncoding: encoding
+				      lossy: false
+				   insecure: true];
 }
 
 - (const char *)UTF8String
 {
-	return [self cStringWithEncoding: OFStringEncodingUTF8];
+	return [self of_cStringWithEncoding: OFStringEncodingUTF8
+				      lossy: false
+				   insecure: false];
 }
 
 - (size_t)length
@@ -1401,8 +1592,12 @@ OF_SINGLETON_METHODS
 
 		for (size_t i = 0; i < length; i++) {
 			char buffer[4];
-			size_t len = _OFUTF8StringEncode(characters[i], buffer);
+			size_t len;
 
+			if (characters[i] == 0)
+				@throw [OFInvalidArgumentException exception];
+
+			len = _OFUTF8StringEncode(characters[i], buffer);
 			if (len == 0)
 				@throw [OFInvalidEncodingException exception];
 
@@ -1415,10 +1610,12 @@ OF_SINGLETON_METHODS
 	case OFStringEncodingISO8859_2:
 	case OFStringEncodingISO8859_3:
 	case OFStringEncodingISO8859_15:
+	case OFStringEncodingWindows1250:
 	case OFStringEncodingWindows1251:
 	case OFStringEncodingWindows1252:
 	case OFStringEncodingCodepage437:
 	case OFStringEncodingCodepage850:
+	case OFStringEncodingCodepage852:
 	case OFStringEncodingCodepage858:
 	case OFStringEncodingMacRoman:
 	case OFStringEncodingKOI8R:
@@ -1439,8 +1636,7 @@ OF_SINGLETON_METHODS
 	OF_UNRECOGNIZED_SELECTOR
 }
 
-- (void)getCharacters: (OFUnichar *)buffer
-	      inRange: (OFRange)range
+- (void)getCharacters: (OFUnichar *)buffer inRange: (OFRange)range
 {
 	for (size_t i = 0; i < range.length; i++)
 		buffer[i] = [self characterAtIndex: range.location + i];
@@ -1646,9 +1842,10 @@ OF_SINGLETON_METHODS
 
 	if (options & OFJSONRepresentationOptionJSON5) {
 		[JSON replaceOccurrencesOfString: @"\n" withString: @"\\\n"];
+		[JSON replaceOccurrencesOfString: @"\0" withString: @"\\0"];
 
 		if (options & OFJSONRepresentationOptionIsIdentifier) {
-			const char *cString = self.UTF8String;
+			const char *cString = JSON.UTF8String;
 
 			if ((!OFASCIIIsAlpha(cString[0]) &&
 			    cString[0] != '_' && cString[0] != '$') ||
@@ -1662,6 +1859,7 @@ OF_SINGLETON_METHODS
 		}
 	} else {
 		[JSON replaceOccurrencesOfString: @"\n" withString: @"\\n"];
+		[JSON replaceOccurrencesOfString: @"\0" withString: @"\\u0000"];
 
 		[JSON insertString: @"\"" atIndex: 0];
 		[JSON appendString: @"\""];
@@ -1708,7 +1906,8 @@ OF_SINGLETON_METHODS
 	} else
 		@throw [OFOutOfRangeException exception];
 
-	[data addItems: self.UTF8String count: length];
+	[data addItems: [self insecureCStringWithEncoding: OFStringEncodingUTF8]
+		 count: length];
 
 	return data;
 }
@@ -2503,7 +2702,7 @@ OF_SINGLETON_METHODS
 	for (size_t i = 0; i < length; i++) {
 		OFUnichar c = characters[i];
 
-		if (c > 0x10FFFF) {
+		if (c > 0x10FFFF || c == 0) {
 			OFFreeMemory(buffer);
 			@throw [OFInvalidEncodingException exception];
 		}
@@ -2578,9 +2777,13 @@ OF_SINGLETON_METHODS
 		[self getCharacters: buffer inRange: OFMakeRange(0, length)];
 		buffer[length] = 0;
 
-		if (byteOrder != OFByteOrderNative)
-			for (size_t i = 0; i < length; i++)
+		for (size_t i = 0; i < length; i++) {
+			if (buffer[i] == 0)
+				@throw [OFInvalidEncodingException exception];
+
+			if (byteOrder != OFByteOrderNative)
 				buffer[i] = OFByteSwap32(buffer[i]);
+		}
 
 		ret = [[OFData dataWithItemsNoCopy: buffer
 					     count: length + 1
