@@ -33,6 +33,8 @@
 #import "OFArray.h"
 #import "OFData.h"
 #import "OFMutableUTF8String.h"
+#import "OFString.h"
+#import "OFString+Private.h"
 
 #import "OFInitializationFailedException.h"
 #import "OFInvalidArgumentException.h"
@@ -49,6 +51,8 @@ extern const OFChar16 _OFISO8859_3Table[];
 extern const size_t _OFISO8859_3TableOffset;
 extern const OFChar16 _OFISO8859_15Table[];
 extern const size_t _OFISO8859_15TableOffset;
+extern const OFChar16 _OFWindows1250Table[];
+extern const size_t _OFWindows1250TableOffset;
 extern const OFChar16 _OFWindows1251Table[];
 extern const size_t _OFWindows1251TableOffset;
 extern const OFChar16 _OFWindows1252Table[];
@@ -57,6 +61,8 @@ extern const OFChar16 _OFCodepage437Table[];
 extern const size_t _OFCodepage437TableOffset;
 extern const OFChar16 _OFCodepage850Table[];
 extern const size_t _OFCodepage850TableOffset;
+extern const OFChar16 _OFCodepage852Table[];
+extern const size_t _OFCodepage852TableOffset;
 extern const OFChar16 _OFCodepage858Table[];
 extern const size_t _OFCodepage858TableOffset;
 extern const OFChar16 _OFMacRomanTable[];
@@ -86,12 +92,17 @@ memcasecmp(const char *first, const char *second, size_t length)
 }
 
 int
-_OFUTF8StringCheck(const char *UTF8String, size_t UTF8Length, size_t *length)
+_OFUTF8StringCheck(const char *UTF8String, size_t UTF8Length, size_t *length,
+    bool *containsNull)
 {
 	size_t tmpLength = UTF8Length;
 	int isUTF8 = 0;
+	bool tmpContainsNull = false;
 
 	for (size_t i = 0; i < UTF8Length; i++) {
+		if OF_UNLIKELY (UTF8String[i] == '\0')
+			tmpContainsNull = true;
+
 		/* No sign of UTF-8 here */
 		if OF_LIKELY (!(UTF8String[i] & 0x80))
 			continue;
@@ -149,6 +160,9 @@ _OFUTF8StringCheck(const char *UTF8String, size_t UTF8Length, size_t *length)
 	if (length != NULL)
 		*length = tmpLength;
 
+	if (containsNull != NULL)
+		*containsNull = tmpContainsNull;
+
 	return isUTF8;
 }
 
@@ -212,7 +226,7 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 		_s->cStringLength = UTF8StringLength;
 
 		switch (_OFUTF8StringCheck(UTF8String, UTF8StringLength,
-		    &_s->length)) {
+		    &_s->length, &_s->containsNull)) {
 		case 1:
 			_s->isUTF8 = true;
 			break;
@@ -256,7 +270,7 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 		if (encoding == OFStringEncodingUTF8 ||
 		    encoding == OFStringEncodingASCII) {
 			switch (_OFUTF8StringCheck(cString, cStringLength,
-			    &_s->length)) {
+			    &_s->length, &_s->containsNull)) {
 			case 1:
 				if (encoding == OFStringEncodingASCII)
 					@throw [OFInvalidEncodingException
@@ -289,9 +303,12 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 				}
 
 				_s->isUTF8 = true;
+
+				if OF_UNLIKELY (cString[i] == '\0')
+					_s->containsNull = true;
+
 				bytes = _OFUTF8StringEncode(
 				    (uint8_t)cString[i], buffer);
-
 				if (bytes == 0)
 					@throw [OFInvalidEncodingException
 					    exception];
@@ -324,6 +341,9 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 #ifdef HAVE_ISO_8859_15
 		CASE(OFStringEncodingISO8859_15, _OFISO8859_15Table)
 #endif
+#ifdef HAVE_WINDOWS_1250
+		CASE(OFStringEncodingWindows1250, _OFWindows1250Table)
+#endif
 #ifdef HAVE_WINDOWS_1251
 		CASE(OFStringEncodingWindows1251, _OFWindows1251Table)
 #endif
@@ -336,6 +356,9 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 #ifdef HAVE_CODEPAGE_850
 		CASE(OFStringEncodingCodepage850, _OFCodepage850Table)
 #endif
+#ifdef HAVE_CODEPAGE_852
+		CASE(OFStringEncodingCodepage852, _OFCodepage852Table)
+#endif
 #ifdef HAVE_CODEPAGE_858
 		CASE(OFStringEncodingCodepage858, _OFCodepage858Table)
 #endif
@@ -360,19 +383,25 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 			char buffer[4];
 			size_t byteLength;
 
-			if (character < tableOffset) {
-				_s->cString[j++] = cString[i];
-				continue;
-			}
-
-			unichar = table[character - tableOffset];
+			if (character >= tableOffset)
+				unichar = table[character - tableOffset];
+			else
+				unichar = character;
 
 			if (unichar == 0xFFFF)
 				@throw [OFInvalidEncodingException exception];
 
-			_s->isUTF8 = true;
-			byteLength = _OFUTF8StringEncode(unichar, buffer);
+			if (unichar < 0x7F) {
+				_s->cString[j++] = (char)unichar;
+				continue;
+			}
 
+			_s->isUTF8 = true;
+
+			if OF_UNLIKELY (cString[i] == '\0')
+				_s->containsNull = true;
+
+			byteLength = _OFUTF8StringEncode(unichar, buffer);
 			if (byteLength == 0)
 				@throw [OFInvalidEncodingException exception];
 
@@ -418,7 +447,7 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 		}
 
 		switch (_OFUTF8StringCheck(UTF8String, UTF8StringLength,
-		    &_s->length)) {
+		    &_s->length, &_s->containsNull)) {
 		case 1:
 			_s->isUTF8 = true;
 			break;
@@ -445,18 +474,29 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 		_s = &_storage;
 
 		_s->cStringLength = string.UTF8StringLength;
-
-		if ([string isKindOfClass: [OFUTF8String class]] ||
-		    [string isKindOfClass: [OFMutableUTF8String class]])
-			_s->isUTF8 = ((OFUTF8String *)string)->_s->isUTF8;
-		else
-			_s->isUTF8 = true;
-
 		_s->length = string.length;
 
 		_s->cString = OFAllocMemory(_s->cStringLength + 1, 1);
-		memcpy(_s->cString, string.UTF8String, _s->cStringLength + 1);
+		memcpy(_s->cString,
+		    [string insecureCStringWithEncoding: OFStringEncodingUTF8],
+		    _s->cStringLength + 1);
 		_s->freeWhenDone = true;
+
+		if ([string isKindOfClass: [OFUTF8String class]] ||
+		    [string isKindOfClass: [OFMutableUTF8String class]]) {
+			_s->isUTF8 = ((OFUTF8String *)string)->_s->isUTF8;
+			_s->containsNull =
+			    ((OFUTF8String *)string)->_s->containsNull;
+		} else {
+			switch (_OFUTF8StringCheck(_s->cString,
+			    _s->cStringLength, NULL, &_s->containsNull)) {
+			case 1:
+				_s->isUTF8 = true;
+				break;
+			case -1:
+				@throw [OFInvalidEncodingException exception];
+			}
+		}
 	} @catch (id e) {
 		[self release];
 		@throw e;
@@ -490,6 +530,9 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 			if (len > 1)
 				_s->isUTF8 = true;
 
+			if OF_UNLIKELY (characters[i] == 0)
+				_s->containsNull = true;
+
 			j += len;
 		}
 
@@ -575,6 +618,9 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 			if (len > 1)
 				_s->isUTF8 = true;
 
+			if OF_UNLIKELY (character == 0)
+				_s->containsNull = true;
+
 			j += len;
 		}
 
@@ -623,10 +669,9 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 		j = 0;
 		for (size_t i = 0; i < length; i++) {
 			char buffer[4];
-			size_t len = _OFUTF8StringEncode((swap
-			    ? OFByteSwap32(characters[i])
-			    : characters[i]),
-			    buffer);
+			OFUnichar character = (swap
+			    ? OFByteSwap32(characters[i]) : characters[i]);
+			size_t len = _OFUTF8StringEncode(character, buffer);
 
 			switch (len) {
 			case 1:
@@ -644,6 +689,9 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 			default:
 				@throw [OFInvalidEncodingException exception];
 			}
+
+			if OF_UNLIKELY (character == 0)
+				_s->containsNull = true;
 		}
 
 		_s->cString[j] = '\0';
@@ -684,7 +732,7 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 
 		@try {
 			switch (_OFUTF8StringCheck(tmp, cStringLength,
-			    &_s->length)) {
+			    &_s->length, &_s->containsNull)) {
 			case 1:
 				_s->isUTF8 = true;
 				break;
@@ -718,6 +766,9 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 	   maxLength: (size_t)maxLength
 	    encoding: (OFStringEncoding)encoding
 {
+	if (_s->containsNull)
+		@throw [OFInvalidEncodingException exception];
+
 	switch (encoding) {
 	case OFStringEncodingASCII:
 		if (_s->isUTF8)
@@ -739,6 +790,9 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 
 - (const char *)cStringWithEncoding: (OFStringEncoding)encoding
 {
+	if (_s->containsNull)
+		@throw [OFInvalidEncodingException exception];
+
 	switch (encoding) {
 	case OFStringEncodingASCII:
 		if (_s->isUTF8)
@@ -751,8 +805,25 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 	}
 }
 
+- (const char *)insecureCStringWithEncoding: (OFStringEncoding)encoding
+{
+	switch (encoding) {
+	case OFStringEncodingASCII:
+		if (_s->isUTF8)
+			@throw [OFInvalidEncodingException exception];
+		/* intentional fall-through */
+	case OFStringEncodingUTF8:
+		return _s->cString;
+	default:
+		return [super insecureCStringWithEncoding: encoding];
+	}
+}
+
 - (const char *)UTF8String
 {
+	if (_s->containsNull)
+		@throw [OFInvalidEncodingException exception];
+
 	return _s->cString;
 }
 
@@ -798,7 +869,9 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 	    _s->hasHash && string->_s->hasHash && _s->hash != string->_s->hash)
 		return false;
 
-	if (strcmp(_s->cString, string.UTF8String) != 0)
+	if (memcmp(_s->cString,
+	    [string insecureCStringWithEncoding: OFStringEncodingUTF8],
+	    _s->cStringLength) != 0)
 		return false;
 
 	return true;
@@ -819,7 +892,8 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 	minimumCStringLength = (_s->cStringLength > otherCStringLength
 	    ? otherCStringLength : _s->cStringLength);
 
-	if ((compare = memcmp(_s->cString, string.UTF8String,
+	if ((compare = memcmp(_s->cString,
+	    [string insecureCStringWithEncoding: OFStringEncodingUTF8],
 	    minimumCStringLength)) == 0) {
 		if (_s->cStringLength > otherCStringLength)
 			return OFOrderedDescending;
@@ -846,7 +920,8 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 	if (string == self)
 		return OFOrderedSame;
 
-	otherCString = string.UTF8String;
+	otherCString =
+	    [string insecureCStringWithEncoding: OFStringEncodingUTF8];
 	otherCStringLength = string.UTF8StringLength;
 
 #ifdef OF_HAVE_UNICODE_TABLES
@@ -990,7 +1065,8 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 		 options: (OFStringSearchOptions)options
 		   range: (OFRange)range
 {
-	const char *cString = string.UTF8String;
+	const char *cString =
+	    [string insecureCStringWithEncoding: OFStringEncodingUTF8];
 	size_t cStringLength = string.UTF8StringLength;
 	size_t rangeLocation, rangeLength;
 
@@ -1048,7 +1124,8 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 
 - (bool)containsString: (OFString *)string
 {
-	const char *cString = string.UTF8String;
+	const char *cString =
+	    [string insecureCStringWithEncoding: OFStringEncodingUTF8];
 	size_t cStringLength = string.UTF8StringLength;
 
 	if (cStringLength == 0)
@@ -1090,7 +1167,9 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 	if (cStringLength > _s->cStringLength)
 		return false;
 
-	return (memcmp(_s->cString, prefix.UTF8String, cStringLength) == 0);
+	return (memcmp(_s->cString,
+	    [prefix insecureCStringWithEncoding: OFStringEncodingUTF8],
+	    cStringLength) == 0);
 }
 
 - (bool)hasSuffix: (OFString *)suffix
@@ -1101,7 +1180,8 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 		return false;
 
 	return (memcmp(_s->cString + (_s->cStringLength - cStringLength),
-	    suffix.UTF8String, cStringLength) == 0);
+	    [suffix insecureCStringWithEncoding: OFStringEncodingUTF8],
+	    cStringLength) == 0);
 }
 
 - (OFArray *)componentsSeparatedByString: (OFString *)delimiter
@@ -1123,7 +1203,7 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 
 	array = [OFMutableArray array];
 	pool = objc_autoreleasePoolPush();
-	cString = delimiter.UTF8String;
+	cString = [delimiter insecureCStringWithEncoding: OFStringEncodingUTF8];
 	cStringLength = delimiter.UTF8StringLength;
 
 	if (cStringLength > _s->cStringLength) {
@@ -1194,10 +1274,15 @@ _OFUTF8StringIndexToPosition(const char *string, size_t idx, size_t length)
 
 - (const OFChar32 *)UTF32StringWithByteOrder: (OFByteOrder)byteOrder
 {
-	OFChar32 *buffer = OFAllocMemory(_s->length + 1, sizeof(OFChar32));
+	OFChar32 *buffer;
 	size_t i = 0, j = 0;
 	const OFChar32 *ret;
 
+	if (_s->containsNull)
+		@throw [OFInvalidEncodingException exception];
+
+	buffer = OFAllocMemory(_s->length + 1, sizeof(OFChar32));
+
 	while (i < _s->cStringLength) {
 		OFChar32 c;
 		ssize_t cLen;
