@@ -77,6 +77,8 @@
 # import "OFFile.h"
 #endif
 #import "OFINIFile.h"
+#import "OFEmbeddedIRIHandler.h"
+#import "OFINICategory.h"
 #import "OFSettings.h"
 #ifdef OF_HAVE_SOCKETS
 # import "OFStreamSocket.h"
@@ -90,8 +92,12 @@
 # import "OFDNSResourceRecord.h"
 # import "OFDNSResponse.h"
 # import "OFDNSResolver.h"
+# ifdef OF_HAVE_SCTP
+#  import "OFSCTPSocket.h"
+# endif
 # ifdef OF_HAVE_UNIX_SOCKETS
 #  import "OFUNIXDatagramSocket.h"
+#  import "OFUNIXSequencedPacketSocket.h"
 #  import "OFUNIXStreamSocket.h"
 # endif
 # ifdef OF_HAVE_IPX
@@ -153,7 +159,9 @@
 #import "OFAllocFailedException.h"
 #import "OFAlreadyOpenException.h"
 #import "OFException.h"
-#import "OFChangeCurrentDirectoryFailedException.h"
+#ifdef OF_HAVE_FILES
+# import "OFChangeCurrentDirectoryFailedException.h"
+#endif
 #import "OFChecksumMismatchException.h"
 #import "OFCopyItemFailedException.h"
 #import "OFCreateDirectoryFailedException.h"
