@@ -21,7 +21,7 @@
 
 #import "OTTestCase.h"
 
-@implementation OTTestCase: OFObject
+@implementation OTTestCase
 + (OFArray OF_GENERIC(OFPair OF_GENERIC(OFString *, id) *) *)summary
 {
 	return nil;
